﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace howto_generate_with_frequency
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The letter frequencies. See:
        // http://en.wikipedia.org/wiki/Letter_frequency
        private float[] Frequencies =
        {
            8.167f, 1.492f, 2.782f, 4.253f, 12.702f, 
            2.228f, 2.015f, 6.094f, 6.966f, 0.153f, 
            0.772f, 4.025f, 2.406f, 6.749f, 7.507f, 
            1.929f, 0.095f, 5.987f, 6.327f, 9.056f, 
            2.758f, 0.978f, 2.360f, 0.150f, 1.974f, 
            0.074f
        };

        // Random number generator.
        private Random Rand = new Random();

        // The ASCII value of A.
        private int int_A = (int)'A';

        // Make sure the frequencies add up to 100.
        private void Form1_Load(object sender, EventArgs e)
        {
            // Give any difference to E.
            float total = Frequencies.Sum();
            float diff = 100f - total;
            Frequencies[(int)'E' - int_A] += diff;
        }

        // Generate random letters with the indicated frequencies.
        private void btnGenerate_Click(object sender, EventArgs e)
        {
            // Keep track of the number of each letter generated.
            int[] counts = new int[26];

            // Generate the letters.
            int num_letters = int.Parse(txtNumLetters.Text);
            string result = "";
            for (int i = 0; i < num_letters; i++)
            {
                // Generate a number between 0 and 100.
                double num = 100.0 * Rand.NextDouble();

                // See which letter this represents.
                for (int letter_num = 0; ; letter_num++)
                {
                    // Subtract this letter's frequency from num.
                    num -= Frequencies[letter_num];

                    // If num <= 0, then this is the letter.
                    if ((num <= 0) || (letter_num == 25))
                    {
                        char ch = (char)(int_A + letter_num);
                        result += ch.ToString() + ' ';
                        counts[letter_num]++;
                        break;
                    }
                }
            }

            txtLetters.Text = result;
            txtLetters.Select(0, 0);

            // Display the frequencies.
            lstFrequencies.Items.Clear();
            for (int i = 0; i < counts.Length; i++)
            {
                char ch = (char)(int_A + i);
                float frequency = (float)counts[i] / num_letters * 100;
                string str = string.Format("{0}\t{1,6}\t{2,6}\t{3,6}",
                    ch.ToString(),
                    frequency.ToString("0.000"),
                    Frequencies[i].ToString("0.000"),
                    (frequency - Frequencies[i]).ToString("0.000"));
                lstFrequencies.Items.Add(str);
            }
        }
    }
}
